IF exists (select 1 from sysobjects where name='DBA_PM_PlanCache' and type='p')
   DROP procedure DBA_PM_PlanCache
GO

  
CREATE Procedure DBA_PM_PlanCache (@dbname varchar(200))       
As    
BEGIN        
	set nocount on    
	
	--this is called from DBA_PM_Addin_PlanCache which builds a cursor to pass in the db    
	
	DECLARE @SQL VARCHAR(MAX)
	     
	-- Tells you which cached stored procedures are called the most often
	-- This helps you characterize and baseline your workload
	-- Top Cached SPs By Execution Count (SQL 2008)
	SELECT @SQL='
	INSERT INTO SMSPHsfu.dbo.DBA_PM_PlanCacheMetrics
	SELECT TOP(25) 
		getdate() AS CurrTime,
		@@servername AS ServerName,
		''' + @dbname + ''' AS DBName,
		p.name AS [SPName], 
		qs.execution_count AS ExecutionCount,
		ISNULL(qs.execution_count/DATEDIFF(Second, qs.cached_time, GETDATE()), 0) AS [CallsPerSecond],
		qs.total_worker_time AS [TotalWorkerTime],  
		qs.total_elapsed_time AS TotalElapsedTime, 
		qs.total_elapsed_time/qs.execution_count AS [AvgElapsedTime],
		qs.total_worker_time/qs.execution_count AS AvgWorkerTime,
		NULL AS TotalLogicalReads,
		NULL AS AvgLogicalReads,
		NULL AS TotalPhysicalReads,
		NULL AS AvgPhysicalReads,
		NULL AS TotalLogicalWrites,
		NULL AS AvgLogicalWrites,
		qs.cached_time AS CachedTime,
		ROW_NUMBER() OVER (ORDER BY qs.execution_count DESC) AS RowNumber,
		''Top Cached SPs By Execution Count'' AS Metric
	FROM ' + quotename(@dbname) + '.sys.procedures AS p WITH (NOLOCK)
	INNER JOIN sys.dm_exec_procedure_stats AS qs WITH (NOLOCK)
	ON p.[object_id] = qs.[object_id]
	WHERE qs.database_id = DB_ID(''' + @dbname + ''')
	ORDER BY qs.execution_count DESC 
	OPTION (RECOMPILE);
	'
	exec (@SQL) 
	
	-- Top Cached SPs By Avg Elapsed Time (SQL 2008)
	-- This helps you find long-running cached stored procedures 
	SELECT @SQL='
	INSERT INTO SMSPHsfu.dbo.DBA_PM_PlanCacheMetrics
	SELECT TOP(25) 
		getdate() AS CurrTime,
		@@servername AS ServerName,
		''' + @dbname + ''' AS DBName,
		p.name AS [SPName], 
		qs.execution_count,
		ISNULL(qs.execution_count/DATEDIFF(Second, qs.cached_time, GETDATE()), 0) AS [Calls/Second], 
		qs.total_worker_time AS [TotalWorkerTime], 
		qs.total_elapsed_time, 
		qs.total_elapsed_time/qs.execution_count AS [avg_elapsed_time], 
		qs.total_worker_time/qs.execution_count AS [AvgWorkerTime],
		NULL AS TotalLogicalReads,
		NULL AS AvgLogicalReads, 
		NULL AS TotalPhysicalReads,
		NULL AS AvgPhysicalReads,
		NULL AS TotalLogicalWrites,
		NULL AS AvgLogicalWrites,
		qs.cached_time,
		ROW_NUMBER() OVER (ORDER BY qs.total_elapsed_time/qs.execution_count DESC) AS RowNumber,
		''Top Cached SPs By Avg Elapsed Time'' AS Metric
	FROM ' + quotename(@dbname) + '.sys.procedures AS p WITH (NOLOCK)
	INNER JOIN sys.dm_exec_procedure_stats AS qs WITH (NOLOCK)
	ON p.[object_id] = qs.[object_id]
	WHERE qs.database_id = DB_ID(''' + @dbname + ''')
	ORDER BY avg_elapsed_time DESC OPTION (RECOMPILE);
	'
	exec (@SQL) 
	
	
	-- Top Cached SPs By Total Worker time (SQL 2008). Worker time relates to CPU cost
	SELECT @SQL='
	INSERT INTO SMSPHsfu.dbo.DBA_PM_PlanCacheMetrics
	SELECT TOP(25) 
		getdate() AS CurrTime,
		@@servername AS ServerName,
		''' + @dbname + ''' AS DBName,
		p.name AS [SP Name], 
		qs.execution_count, 
		ISNULL(qs.execution_count/DATEDIFF(Second, qs.cached_time, GETDATE()), 0) AS [Calls/Second],
		qs.total_worker_time AS [TotalWorkerTime],
		qs.total_elapsed_time,
		qs.total_elapsed_time/qs.execution_count AS [avg_elapsed_time],  
		qs.total_worker_time/qs.execution_count AS [AvgWorkerTime], 
		NULL AS TotalLogicalReads,
		NULL AS AvgLogicalReads,
		NULL AS TotalPhysicalReads,
		NULL AS AvgPhysicalReads,
		NULL AS TotalLogicalWrites,
		NULL AS AvgLogicalWrites,
		qs.cached_time,
		ROW_NUMBER() OVER (ORDER BY qs.total_worker_time DESC) AS RowNumber,
		''Top Cached SPs By CPU'' AS Metric
	FROM ' + quotename(@dbname) + '.sys.procedures AS p WITH (NOLOCK)
	INNER JOIN sys.dm_exec_procedure_stats AS qs WITH (NOLOCK)
	ON p.[object_id] = qs.[object_id]
	WHERE qs.database_id = DB_ID(''' + @dbname + ''')
	ORDER BY qs.total_worker_time DESC OPTION (RECOMPILE);
	'
	exec (@SQL) 
	

	-- Top Cached SPs By Total Logical Reads (SQL 2008). Logical reads relate to memory pressure
	SELECT @SQL='
	INSERT INTO SMSPHsfu.dbo.DBA_PM_PlanCacheMetrics
	SELECT TOP(25) 
		getdate() AS CurrTime,
		@@servername AS ServerName,
		''' + @dbname + ''' AS DBName,
		p.name AS [SP Name], 
		qs.execution_count, 
		ISNULL(qs.execution_count/DATEDIFF(Second, qs.cached_time, GETDATE()), 0) AS [Calls/Second], 
		NULL AS TotalWorkerTime,
		qs.total_elapsed_time, 
		qs.total_elapsed_time/qs.execution_count AS [avg_elapsed_time], 
		NULL AS AvgWorkerTime,
		qs.total_logical_reads AS [TotalLogicalReads], 
		qs.total_logical_reads/qs.execution_count AS [AvgLogicalReads],
		NULL AS TotalPhysicalReads,
		NULL AS AvgPhysicalReads,
		NULL AS TotalLogicalWrites,
		NULL AS AvgLogicalWrites,
		qs.cached_time,
		ROW_NUMBER() OVER (ORDER BY qs.total_logical_reads DESC) AS RowNumber,
		''Top Cached SPs By Total Logical Reads'' AS Metric
	FROM ' + quotename(@dbname) + '.sys.procedures AS p WITH (NOLOCK)
	INNER JOIN sys.dm_exec_procedure_stats AS qs WITH (NOLOCK)
	ON p.[object_id] = qs.[object_id]
	WHERE qs.database_id = DB_ID(''' + @dbname + ''')
	ORDER BY qs.total_logical_reads DESC OPTION (RECOMPILE);
	'
	exec (@SQL) 
	

	-- Top Cached SPs By Total Physical Reads (SQL 2008). Physical reads relate to disk I/O pressure
	SELECT @SQL='
	INSERT INTO SMSPHsfu.dbo.DBA_PM_PlanCacheMetrics
	SELECT TOP(25) 
		getdate() AS CurrTime,
		@@servername AS ServerName,
		''' + @dbname + ''' AS DBName,
		p.name AS [SP Name],
		qs.execution_count, 
		NULL AS CallsPerSecond,
		NULL AS TotalWorkerTime,
		qs.total_elapsed_time,
		qs.total_elapsed_time/qs.execution_count AS [avg_elapsed_time], 
		NULL AS AvgWorkerTime,
		qs.total_logical_reads,
		NULL AS AvgLogicalReads,
		qs.total_physical_reads AS [TotalPhysicalReads], 
		qs.total_physical_reads/qs.execution_count AS [AvgPhysicalReads], 
		NULL AS TotalLogicalWrites,
		NULL AS AvgLogicalWrites,
		qs.cached_time ,
		ROW_NUMBER() OVER (ORDER BY qs.total_physical_reads DESC) AS RowNumber,
		''Top Cached SPs By Total Physical Reads'' AS Metric
	FROM ' + quotename(@dbname) + '.sys.procedures AS p WITH (NOLOCK)
	INNER JOIN sys.dm_exec_procedure_stats AS qs WITH (NOLOCK)
	ON p.[object_id] = qs.[object_id]
	WHERE qs.database_id = DB_ID(''' + @dbname + ''')
	AND qs.total_physical_reads > 0
	ORDER BY qs.total_physical_reads DESC, qs.total_logical_reads DESC OPTION (RECOMPILE);
	'
	exec (@SQL) 

	-- Top Cached SPs By Total Logical Writes (SQL 2008). 
	SELECT @SQL='
	INSERT INTO SMSPHsfu.dbo.DBA_PM_PlanCacheMetrics
	SELECT TOP(25) 
		getdate() AS CurrTime,
		@@servername AS ServerName,
		''' + @dbname + ''' AS DBName,
		p.name AS [SP Name], 
		qs.execution_count,
		ISNULL(qs.execution_count/DATEDIFF(Second, qs.cached_time, GETDATE()), 0) AS [Calls/Second],
		NULL AS TotalWorkerTime,
		qs.total_elapsed_time, 
		qs.total_elapsed_time/qs.execution_count AS [avg_elapsed_time], 
		NULL AS AvgWorkerTime,
		NULL AS TotalLogicalReads,
		NULL AS AvgLogicalReads,
		NULL AS TotalPhysicalReads,
		NULL AS AvgPhysicalReads,
		qs.total_logical_writes AS [TotalLogicalWrites], 
		qs.total_logical_writes/qs.execution_count AS [AvgLogicalWrites], 
		qs.cached_time,
		ROW_NUMBER() OVER (ORDER BY qs.total_logical_writes DESC) AS RowNumber,
		''Top Cached SPs By Total Logical Writes'' AS Metric
	FROM ' + quotename(@dbname) + '.sys.procedures AS p WITH (NOLOCK)
	INNER JOIN sys.dm_exec_procedure_stats AS qs WITH (NOLOCK)
	ON p.[object_id] = qs.[object_id]
	WHERE qs.database_id = DB_ID(''' + @dbname + ''')
	ORDER BY qs.total_logical_writes DESC OPTION (RECOMPILE);
	'
	exec (@SQL) 	

END
GO  